@@ -1,7 +1,7 @@
 //C-  -*- C++ -*-
 //C- -------------------------------------------------------------------
 //C- DjView4
-//C- Copyright (c) 2006  Leon Bottou
+//C- Copyright (c) 2006-  Leon Bottou
 //C-
 //C- This software is subject to, and may be distributed under, the
 //C- GNU General Public License, either version 2 of the license,
@@ -15,7 +15,7 @@
 //C- GNU General Public License for more details.
 //C-  ------------------------------------------------------------------
 
-// $Id: qdjview.h,v 1.70 2008/07/31 22:14:31 leonb Exp $
+// $Id: qdjview.h,v 1.78 2009/04/01 13:27:39 leonb Exp $
 
 #ifndef QDJVIEW_H
 #define QDJVIEW_H
@@ -65,8 +65,9 @@ class QStackedLayout;
 class QStatusBar;
 class QStringList;
 class QTimer;
+class QTabWidget;
 class QToolBar;
-class QToolBox;
+
 
 
 #if DDJVUAPI_VERSION < 18
@@ -99,19 +100,23 @@ class QDjView : public QMainWindow
   QDjVuDocument      *getDocument()         { return document; } 
   QString             getDocumentFileName() { return documentFileName; }
   QUrl                getDocumentUrl()      { return documentUrl; }
+  QUrl                getDecoratedUrl();
   ViewerMode          getViewerMode()       { return viewerMode; }
   QString             getShortFileName();
   bool                getFullScreen();
-  
+
+  QString      getArgument(QString key);
   QStringList  parseArgument(QString key, QString val);
   QStringList  parseArgument(QString keyEqualVal);
   void         parseDjVuCgiArguments(QUrl url);
   static QUrl  removeDjVuCgiArguments(QUrl url);
-  
+
   int         pageNum(void);
   QString     pageName(int pageno, bool titleonly=false);
   int         pageNumber(QString name, int from = -1);
   QDjView    *copyWindow(bool openDocument=true);
+  int         visibleSideBarTabs();
+  int         hiddenSideBarTabs();
   bool        saveTextFile(QString text, QString filename=QString());
   bool        saveImageFile(QImage image, QString filename=QString());
   bool        startBrowser(QUrl url);
@@ -134,9 +139,10 @@ public slots:
   void  setPageLabelText(QString s = QString());
   void  setMouseLabelText(QString s = QString());
   void  statusMessage(QString s = QString());
-  bool  showSideBar(Qt::DockWidgetAreas areas, int tab=-1);
+  bool  showSideBar(Qt::DockWidgetAreas areas, int tabs);
   bool  showSideBar(QString args, QStringList &errors);
   bool  showSideBar(QString args);
+  bool  showSideBar(bool);
   void  print(void);
   void  saveAs(void);
   void  exportAs(void);
@@ -150,6 +156,7 @@ signals:
   void  documentReady(QDjVuDocument *doc);
   void  pluginStatusMessage(QString message = QString());
   void  pluginGetUrl(QUrl url, QString target);
+  void  pluginOnChange();
 
 protected:
   typedef QDjVuWidget::Position Position;
@@ -167,7 +174,7 @@ protected:
   Saved   *getSavedPrefs(void);
   void     enableContextMenu(bool);
   void     enableScrollBars(bool);
-  void     applyOptions(void);
+  void     applyOptions(bool remember=true);
   void     updateOptions(void);
   void     applySaved(Saved *saved);
   void     updateSaved(Saved *saved);
@@ -256,11 +263,12 @@ protected:
   QComboBox          *modeCombo;
   QComboBox          *zoomCombo;
   QComboBox          *pageCombo;
-  QDockWidget        *sideBar;
-  QToolBox           *sideToolBox;
   QDjViewOutline     *outlineWidget;
+  QDockWidget        *outlineDock;
   QDjViewThumbnails  *thumbnailWidget;
+  QDockWidget        *thumbnailDock;
   QDjViewFind        *findWidget;
+  QDockWidget        *findDock;
   // document data
   QDjVuContext           &djvuContext;
   QDjVuDocument          *document;
@@ -270,7 +278,6 @@ protected:
   bool                    hasNumericalPageTitle;
   // delayed settings
   typedef QPair<QString,QString> StringPair;
-  QString           pendingUrl;
   QString           pendingPage;
   QList<double>     pendingPosition;
   QList<StringPair> pendingHilite;
@@ -329,6 +336,7 @@ protected:
   QAction *actionDisplayBW;
   QAction *actionDisplayForeground;
   QAction *actionDisplayBackground;
+  QAction *actionDisplayHiddenText;
   QAction *actionPreferences;
   QAction *actionViewToolBar;
   QAction *actionViewSideBar;
@@ -349,6 +357,7 @@ protected:
   Saved fsSavedNormal;
   Saved fsSavedFullScreen;
   Qt::WindowStates fsWindowState;
+  QByteArray savedDockState;
   // undo/redo
   class UndoRedo 
   {
