@@ -1,7 +1,7 @@
 //C-  -*- C++ -*-
 //C- -------------------------------------------------------------------
 //C- DjView4
-//C- Copyright (c) 2006  Leon Bottou
+//C- Copyright (c) 2006-  Leon Bottou
 //C-
 //C- This software is subject to, and may be distributed under, the
 //C- GNU General Public License, either version 2 of the license,
@@ -15,7 +15,7 @@
 //C- GNU General Public License for more details.
 //C-  ------------------------------------------------------------------
 
-// $Id: qdjviewexporters.cpp,v 1.26 2008/08/05 06:03:37 leonb Exp $
+// $Id: qdjviewexporters.cpp,v 1.30 2009/02/22 16:50:54 leonb Exp $
 
 #if AUTOCONF
 # include "config.h"
@@ -1566,13 +1566,13 @@ QDjViewTiffExporter::doPage()
       TIFFSetField(tiff, TIFFTAG_YRESOLUTION, (float)(dpi));
       TIFFSetField(tiff, TIFFTAG_PLANARCONFIG, PLANARCONFIG_CONTIG);
       TIFFSetField(tiff, TIFFTAG_ORIENTATION, ORIENTATION_TOPLEFT);
-#if CCITT_SUPPORT
+#ifdef CCITT_SUPPORT
       if (compression != COMPRESSION_CCITT_T6)
 #endif
-#if JPEG_SUPPORT
+#ifdef JPEG_SUPPORT
         if (compression != COMPRESSION_JPEG)
 #endif
-#if ZIP_SUPPORT
+#ifdef ZIP_SUPPORT
           if (compression != COMPRESSION_DEFLATE)
 #endif
             TIFFSetField(tiff, TIFFTAG_ROWSPERSTRIP, (uint32)64);
@@ -1585,8 +1585,10 @@ QDjViewTiffExporter::doPage()
       } else {
         TIFFSetField(tiff, TIFFTAG_BITSPERSAMPLE, (uint16)8);
         TIFFSetField(tiff, TIFFTAG_COMPRESSION, compression);
+#ifdef JPEG_SUPPORT
         if (compression == COMPRESSION_JPEG)
           TIFFSetField(tiff, TIFFTAG_JPEGQUALITY, quality);
+#endif
         if (style == DDJVU_FORMAT_GREY8) {
           TIFFSetField(tiff, TIFFTAG_SAMPLESPERPIXEL, (uint16)1);
           TIFFSetField(tiff, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_MINISBLACK);
@@ -1603,21 +1605,23 @@ QDjViewTiffExporter::doPage()
         }
       }
       // render and save
+      char white = 0xff;
       int rowsize = rect.w * 3;
-      if (style == DDJVU_FORMAT_MSBTOLSB)
+      if (style == DDJVU_FORMAT_MSBTOLSB) {
+        white = 0x00;
         rowsize = (rect.w + 7) / 8;
-      else if (style == DDJVU_FORMAT_GREY8)
+      } else if (style == DDJVU_FORMAT_GREY8)
         rowsize = rect.w;
       if (! (image = (char*)malloc(rowsize * rect.h)))
         message = tr("Out of memory.");
-      else if (! ddjvu_page_render(*page, mode, &rect, &rect, 
-                                   fmt, rowsize, image))
-        message = tr("Cannot render image");
       else if (rowsize != TIFFScanlineSize(tiff))
         message = tr("Internal error.");
-      else
+      else 
         {
           char *s = image;
+          ddjvu_rect_t *r = &rect;
+          if (! ddjvu_page_render(*page, mode, r, r, fmt, rowsize, image))
+            memset(image, white, rowsize * rect.h);
           for (int i=0; i<(int)rect.h; i++, s+=rowsize)
             TIFFWriteScanline(tiff, s, i, 0);
         }
