@@ -1,7 +1,7 @@
 //C-  -*- C++ -*-
 //C- -------------------------------------------------------------------
 //C- DjView4
-//C- Copyright (c) 2006  Leon Bottou
+//C- Copyright (c) 2006-  Leon Bottou
 //C-
 //C- This software is subject to, and may be distributed under, the
 //C- GNU General Public License, either version 2 of the license,
@@ -15,7 +15,7 @@
 //C- GNU General Public License for more details.
 //C-  ------------------------------------------------------------------
 
-// $Id: qdjviewplugin.cpp,v 1.43 2008/07/31 15:24:42 leonb Exp $
+// $Id: qdjviewplugin.cpp,v 1.54 2009/02/22 22:37:30 leonb Exp $
 
 #if AUTOCONF
 # include "config.h"
@@ -108,11 +108,11 @@ public slots:
   void containerClosed();
   void showStatus(QString message);
   void getUrl(QUrl url, QString target);
+  void onChange();
   void quit();
   void dispatch();
   void lastViewerClosed();
   void continueExec();
-  
 };
 
 
@@ -146,6 +146,7 @@ struct QDjViewPlugin::Instance
   QStringList                    args;
   QByteArray                     saved;
   int                            savedformat;
+  int                            onchange;
   QDjView::ViewerMode            viewerMode;
   ~Instance();
   Instance(QDjViewPlugin *dispatcher);
@@ -355,6 +356,16 @@ QDjViewPlugin::Forwarder::getUrl(QUrl url, QString target)
 }
 
 
+void 
+QDjViewPlugin::Forwarder::onChange()
+{
+  QDjView *viewer = qobject_cast<QDjView*>(sender());
+  Instance *instance = dispatcher->findInstance(viewer);
+  if (instance)
+    dispatcher->onChange(instance);
+}
+
+
 void 
 QDjViewPlugin::Forwarder::quit()
 {
@@ -381,6 +392,26 @@ QDjViewPlugin::Forwarder::continueExec()
   dispatcher->continueExec();
 }
 
+
+#if HAVE_X11
+static bool x11EventFilter(void *message, long *)
+{
+  // We define a low level handler because we need to make
+  // the window active before calling the QX11EmbedWidget event filter.
+  XEvent *event = reinterpret_cast<XEvent*>(message);
+  if (event->type == ButtonPress)
+    {
+      QWidget *w = QWidget::find(event->xbutton.window);
+      if (w && w->window()->objectName() == "djvu_shell")
+        {
+          QApplication *app = (QApplication*)QCoreApplication::instance();
+          app->setActiveWindow(w->window());
+        }
+    }
+  return false;
+}
+#endif
+
 bool 
 QDjViewPlugin::Forwarder::eventFilter(QObject *o, QEvent *e)
 {
@@ -389,18 +420,9 @@ QDjViewPlugin::Forwarder::eventFilter(QObject *o, QEvent *e)
       QWidget *w = static_cast<QWidget*>(o);
       switch( e->type() )
         {
-          // Send keyboard events to last clicked window.
-        case QEvent::MouseButtonPress:
-          if (!dispatcher->xembedFlag)
-            {
-              QWidget *window = w->window();
-              if (window->objectName() == "djvu_shell")
-                dispatcher->application->setActiveWindow(window);
-            }
-          break;
+#ifdef Q_WS_X11
           // Try to fix transient windows properties.
           // - this does not work too well...
-#ifdef Q_WS_X11
         case QEvent::Show:
           if (w->windowFlags() & Qt::Window)
             QApplication::postEvent(w, new QEvent(QEvent::User));
@@ -431,13 +453,16 @@ QDjViewPlugin::Instance::~Instance()
   destroy();
   delete shell;
   delete djview;
-  delete document;
+  if (document)
+    document->deref();
+  document = 0;
 }
 
 
 QDjViewPlugin::Instance::Instance(QDjViewPlugin *parent)
   : url(), dispatcher(parent), 
-    document(0), shell(0), djview(0), container(0)
+    document(0), shell(0), djview(0), container(0),
+    onchange(0)
 {
 }
 
@@ -445,16 +470,19 @@ QDjViewPlugin::Instance::Instance(QDjViewPlugin *parent)
 void
 QDjViewPlugin::Instance::open()
 {
-  if (!document && url.isValid() && djview)
+  if (!document && url.isValid())
     {
       document = new QDjViewPlugin::Document(this);
+      document->ref();
+    }
+  if (document && djview && !djview->getDocument())
+    {
       djview->open(document, url);
       restore(djview->getDjVuWidget());
       shell->show();
     }
 }
 
-
 void
 QDjViewPlugin::Instance::destroy()
 {
@@ -579,7 +607,9 @@ enum {
   CMD_GET_URL_NOTIFY = 12,
   CMD_URL_NOTIFY = 13,
   CMD_HANDSHAKE = 14,
-  CMD_SET_OPTION = 15,
+  CMD_SET_DJVUOPT = 15,
+  CMD_GET_DJVUOPT = 16,
+  CMD_ON_CHANGE = 17,
 };
 
 #define OK_STRING   "OK"
@@ -779,14 +809,6 @@ QDjViewPlugin::cmdNew()
   readString(pipeRead);  // djvuDir (unused)
   int argc = readInteger(pipeRead);
   QStringList args;
-  // skip plugin related keywords
-  QSet<QString> pluginWords;
-  pluginWords << "src" << "type" << "pluginspage"
-              << "pluginurl" << "align" << "border"
-              << "frameborder" << "height" << "width"
-              << "units" << "hidden" << "hspace" 
-              << "vspace" << "name" << "palette";
-  // collect
   for (int i=0; i<argc; i++)
     {
       QString key = readString(pipeRead);
@@ -794,7 +816,7 @@ QDjViewPlugin::cmdNew()
       QString k = key.toLower();
       if (k == "flags")
         args += val.split(QRegExp("\\s+"), QString::SkipEmptyParts);
-      else if (! pluginWords.contains(k))
+      else
         args += key + QString("=") + val;
     }
   
@@ -865,10 +887,11 @@ QDjViewPlugin::cmdAttachWindow()
 #endif
       application = new QDjViewApplication(argc, const_cast<char**>(argv));
       application->setQuitOnLastWindowClosed(false);
-      argc = 1;
 #if HAVE_X11
+      application->setEventFilter(x11EventFilter);
       XCloseDisplay(dpy);
 #endif
+      argc = 1;
       context = application->djvuContext();
       forwarder = new Forwarder(this);
       application->installEventFilter(forwarder);
@@ -900,7 +923,7 @@ QDjViewPlugin::cmdAttachWindow()
           shell = embed;
           QObject::connect(shell, SIGNAL(containerClosed()),
                            forwarder, SLOT(containerClosed()) );
-          shell->setObjectName("djvu_xembed");
+          shell->setObjectName("djvu_shell");
           shell->setGeometry(0, 0, width, height);
           embed->embedInto(window);
         }
@@ -928,16 +951,15 @@ QDjViewPlugin::cmdAttachWindow()
                        forwarder, SLOT(showStatus(QString)) );
       QObject::connect(djview, SIGNAL(pluginGetUrl(QUrl,QString)),
                        forwarder, SLOT(getUrl(QUrl,QString)) );
+      QObject::connect(djview, SIGNAL(pluginOnChange()),
+                       forwarder, SLOT(onChange()) );
       instance->shell = shell;
       instance->djview = djview;
       instance->container = window;
       // apply arguments
-      QStringList errors;
-      foreach (QString s, instance->args)
-        errors << djview->parseArgument(s);
-      if (errors.size() > 0)
-        foreach(QString error, errors)
-          qWarning((const char*)error.toLocal8Bit());
+      while (instance->args.size() > 0)
+        djview->parseArgument(instance->args.takeFirst());
+      instance->args.clear();
     }
   // map and reparent djview object
   instance->open();
@@ -1040,7 +1062,8 @@ QDjViewPlugin::cmdNewStream()
         {
           instance->url = url;
           url = QDjView::removeDjVuCgiArguments(url);
-          new QDjViewPlugin::Stream(0, url, instance);
+          Stream *s = new QDjViewPlugin::Stream(0, url, instance);
+          s->checked = true;
           instance->open();
         }
       // search stream
@@ -1151,7 +1174,7 @@ QDjViewPlugin::cmdHandshake()
 
 
 void
-QDjViewPlugin::cmdSetOption()
+QDjViewPlugin::cmdSetDjVuOpt()
 {
   // protocol extension for plugin scripting
   Instance *instance = (Instance*) readPointer(pipeRead);
@@ -1164,9 +1187,48 @@ QDjViewPlugin::cmdSetOption()
       writeString(pipeWrite, QByteArray(ERR_STRING));
       return;
     }
+  // return code
+  writeString(pipeWrite, QByteArray(OK_STRING));
   // apply options
+  QStringList errors;
   if (instance->djview)
-    instance->djview->parseArgument(key, value);
+    errors = instance->djview->parseArgument(key, value);
+  else
+    instance->args += key + QString("=") + value;
+  // print error messages
+  if (errors.size() > 0)
+    foreach(QString error, errors)
+      qWarning((const char*)error.toLocal8Bit());
+}
+
+
+void
+QDjViewPlugin::cmdGetDjVuOpt()
+{
+  // protocol extension for plugin scripting.
+  Instance *instance = (Instance*) readPointer(pipeRead);
+  QString key = readString(pipeRead);
+  // check instance.
+  if (!instances.contains(instance))
+    {
+      fprintf(stderr, "djview dispatcher: bad instance\n");
+      writeString(pipeWrite, QByteArray(ERR_STRING));
+      return;
+    }
+  // get argument.
+  QString value;
+  if (instance->djview)
+    value = instance->djview->getArgument(key);
+  writeString(pipeWrite, QByteArray(OK_STRING));
+  writeString(pipeWrite, value);
+}
+
+
+void
+QDjViewPlugin::cmdOnChange()
+{
+  Instance *instance = (Instance*) readPointer(pipeRead);
+  instance->onchange = readInteger(pipeRead);
   writeString(pipeWrite, QByteArray(OK_STRING));
 }
 
@@ -1283,8 +1345,13 @@ QDjViewPlugin::exec()
   qWarning("QDjViewPlugin::exec() begin");
   try 
     {
-      // startup message
-      writeString(pipeWrite, QByteArray("DJVIEW/4"));
+      // startup message announcing capabilities
+#if HAVE_QX11EMBED
+      const char *djviewName = "DJVIEW/4 SCRIPT XEMBED";
+#else
+      const char *djviewName = "DJVIEW/4 SCRIPT";
+#endif
+      writeString(pipeWrite, QByteArray(djviewName));
       // dispatch until we get display
       while (!application && !quitFlag)
         dispatch();
@@ -1419,6 +1486,24 @@ QDjViewPlugin::getUrl(Instance *instance, QUrl url, QString target)
 }
 
 
+void
+QDjViewPlugin::onChange(Instance *instance)
+{
+  try
+    {
+      if (!instance->onchange)
+        return;
+      writeInteger(pipeRequest, CMD_ON_CHANGE);
+      writePointer(pipeRequest, (void*) instance);
+    }
+  catch(int err)
+    {
+      reportError(err);
+      exit(10);
+    }
+}
+
+
 void
 QDjViewPlugin::showStatus(Instance *instance, QString message)
 {
@@ -1457,7 +1542,9 @@ QDjViewPlugin::dispatch()
         case CMD_DESTROY_STREAM: cmdDestroyStream(); break;
         case CMD_URL_NOTIFY:     cmdUrlNotify(); break;
         case CMD_HANDSHAKE:      cmdHandshake(); break;
-        case CMD_SET_OPTION:     cmdSetOption(); break;
+        case CMD_SET_DJVUOPT:    cmdSetDjVuOpt(); break;
+        case CMD_GET_DJVUOPT:    cmdGetDjVuOpt(); break;
+        case CMD_ON_CHANGE:      cmdOnChange(); break;
         default:                 throw 3;
         }
     }
