@@ -1,7 +1,7 @@
 //C-  -*- C++ -*-
 //C- -------------------------------------------------------------------
 //C- DjView4
-//C- Copyright (c) 2006  Leon Bottou
+//C- Copyright (c) 2006-  Leon Bottou
 //C-
 //C- This software is subject to, and may be distributed under, the
 //C- GNU General Public License, either version 2 of the license,
@@ -15,7 +15,7 @@
 //C- GNU General Public License for more details.
 //C-  ------------------------------------------------------------------
 
-// $Id: qdjviewprefs.cpp,v 1.49 2008/07/31 16:35:27 leonb Exp $
+// $Id: qdjviewprefs.cpp,v 1.52 2009/04/01 13:19:16 leonb Exp $
 
 #if AUTOCONF
 # include "config.h"
@@ -124,8 +124,7 @@ QDjViewPrefs::instance(void)
 QDjViewPrefs::Saved::Saved(void)
   : remember(true),
     options(defaultOptions),
-    zoom(QDjVuWidget::ZOOM_FITWIDTH),
-    sidebarTab(0)
+    zoom(QDjVuWidget::ZOOM_FITWIDTH)
 {
 }
 
@@ -149,6 +148,7 @@ QDjViewPrefs::QDjViewPrefs(void)
     thumbnailSmart(true),
     searchWordsOnly(true),
     searchCaseSensitive(false),
+    searchRegExpMode(false),
     infoDialogTab(0),
     metaDialogTab(0),
     printerGamma(0.0),
@@ -290,8 +290,6 @@ QDjViewPrefs::loadGroup(QSettings &s, QString name, Saved &saved)
     saved.zoom = s.value("zoom").toInt();
   if (s.contains("state"))
     saved.state = s.value("state").toByteArray();
-  if (s.contains("sidebarTab"))
-    saved.sidebarTab = s.value("sidebarTab").toInt();
   s.endGroup();
   // we always want these options
   saved.options |= (QDjViewPrefs::HANDLE_MOUSE |
@@ -362,6 +360,8 @@ QDjViewPrefs::load()
     searchWordsOnly = s.value("searchWordsOnly").toBool();
   if (s.contains("searchCaseSensitive"))
     searchCaseSensitive = s.value("searchCaseSensitive").toBool();
+  if (s.contains("searchRegExpMode"))
+    searchRegExpMode = s.value("searchRegExpMode").toBool();
   
   if (s.contains("infoDialogTab"))
     infoDialogTab = s.value("infoDialogTab").toInt();
@@ -391,7 +391,6 @@ QDjViewPrefs::saveGroup(QSettings &s, QString name, Saved &saved)
   s.setValue("options", optionsToString(saved.options));
   s.setValue("zoom", saved.zoom);
   s.setValue("state", saved.state);
-  s.setValue("sidebarTab", saved.sidebarTab);
   s.endGroup();
 }
 
@@ -437,6 +436,7 @@ QDjViewPrefs::save(void)
 
   s.setValue("searchWordsOnly", searchWordsOnly);
   s.setValue("searchCaseSensitive", searchCaseSensitive);
+  s.setValue("searchRegExpMode", searchRegExpMode);
   
   s.setValue("infoDialogTab", infoDialogTab);
   s.setValue("metaDialogTab", metaDialogTab);
