@@ -1,7 +1,7 @@
 //C-  -*- C++ -*-
 //C- -------------------------------------------------------------------
 //C- DjView4
-//C- Copyright (c) 2006  Leon Bottou
+//C- Copyright (c) 2006-  Leon Bottou
 //C-
 //C- This software is subject to, and may be distributed under, the
 //C- GNU General Public License, either version 2 of the license,
@@ -15,7 +15,7 @@
 //C- GNU General Public License for more details.
 //C-  ------------------------------------------------------------------
 
-// $Id: qdjviewprefs.h,v 1.40 2008/08/03 15:20:16 leonb Exp $
+// $Id: qdjviewprefs.h,v 1.43 2009/04/01 13:19:16 leonb Exp $
 
 #ifndef QDJVIEWPREFS_H
 #define QDJVIEWPREFS_H
@@ -130,7 +130,6 @@ public:
     Options    options;         //!< Miscellaneous flags.
     int        zoom;            //!< Zoom factor.
     QByteArray state;           //!< Toolbar and sidebar geometry.
-    int        sidebarTab;      //!< Selected sidebar tab. 
   };
 
   // Preferences remembered between program invocations.
@@ -161,6 +160,7 @@ public:
   bool       thumbnailSmart;
   bool       searchWordsOnly;
   bool       searchCaseSensitive;
+  bool       searchRegExpMode;
 
   // Dialog preferences
   int        infoDialogTab;     //!< Active tab in info dialog.
