@@ -1,7 +1,7 @@
 //C-  -*- C++ -*-
 //C- -------------------------------------------------------------------
 //C- DjView4
-//C- Copyright (c) 2006  Leon Bottou
+//C- Copyright (c) 2006-  Leon Bottou
 //C-
 //C- This software is subject to, and may be distributed under, the
 //C- GNU General Public License, either version 2 of the license,
@@ -15,7 +15,7 @@
 //C- GNU General Public License for more details.
 //C-  ------------------------------------------------------------------
 
-// $Id: qdjviewsidebar.cpp,v 1.54 2008/07/31 22:14:31 leonb Exp $
+// $Id: qdjviewsidebar.cpp,v 1.61 2009/04/06 11:15:07 leonb Exp $
 
 #if AUTOCONF
 # include "config.h"
@@ -55,6 +55,7 @@
 #include <QStringList>
 #include <QTimer>
 #include <QToolBar>
+#include <QToolButton>
 #include <QTreeWidget>
 #include <QVariant>
 #include <QVBoxLayout>
@@ -875,7 +876,7 @@ private:
   QTimer *animTimer;
   QTimer *workTimer;
   QItemSelectionModel *selection;
-  QPushButton *animButton;
+  QAbstractButton *animButton;
   QIcon animIcon;
   QIcon findIcon;
   QRegExp find;
@@ -885,6 +886,7 @@ private:
   bool searchBackwards;
   bool caseSensitive;
   bool wordOnly;
+  bool regExpMode;
   bool working;
   bool pending;
 };
@@ -905,6 +907,7 @@ QDjViewFind::Model::Model(QDjViewFind *widget)
     searchBackwards(false),
     caseSensitive(false),
     wordOnly(true),
+    regExpMode(false),
     working(false),
     pending(false)
 {
@@ -1062,7 +1065,7 @@ QDjViewFind::Model::documentClosed(QDjVuDocument *doc)
   widget->eraseText();
   widget->combo->setEnabled(false);
   widget->label->setText(QString());
-  widget->stack->setCurrentIndex(0);
+  widget->stack->setCurrentWidget(widget->label);
 }
 
 
@@ -1108,12 +1111,29 @@ miniexp_get_rect(miniexp_t &r, QRect &rect)
 }
 
 
+static int
+parse_text_type(miniexp_t exp)
+{
+  static const char *names[] = { 
+    "char", "word", "line", "para", "region", "column", "page"
+  };
+  static const int nsymbs = sizeof(names)/sizeof(const char*);
+  static miniexp_t symbs[nsymbs];
+  if (! symbs[0])
+    for (int i=0; i<nsymbs; i++)
+      symbs[i] = miniexp_symbol(names[i]);
+  for (int i=0; i<nsymbs; i++)
+    if (exp == symbs[i])
+      return i;
+  return -1;
+}
+
 static bool
-miniexp_get_text(miniexp_t exp, 
-                 QString &result, QMap<int,miniexp_t> &positions,
-                 QList<miniexp_t> nospace, QString &comma)
+miniexp_get_text(miniexp_t exp, QString &result, 
+                 QMap<int,miniexp_t> &positions, int &state)
 {
   miniexp_t type = miniexp_car(exp);
+  int typenum = parse_text_type(type);
   miniexp_t r = exp = miniexp_cdr(exp);
   if (! miniexp_symbolp(type))
     return false;
@@ -1121,25 +1141,24 @@ miniexp_get_text(miniexp_t exp,
   if (! miniexp_get_rect(r, rect))
     return false;
   miniexp_t s = miniexp_car(r);
+  state = qMax(state, typenum);
   if (miniexp_stringp(s) && !miniexp_cdr(r))
     {
-      result += comma;
+      result += (state >= 2) ? "\n" : (state >= 1) ? " " : "";
+      state = -1;
       positions[result.size()] = exp;
       result += QString::fromUtf8(miniexp_to_str(s));
-      if (comma.isEmpty() && !nospace.contains(type))
-        comma = " ";
-      return true;
+      r = miniexp_cdr(r);
     }
   while(miniexp_consp(s))
     {
-      miniexp_get_text(s, result, positions, nospace, comma);
+      miniexp_get_text(s, result, positions, state);
       r = miniexp_cdr(r);
       s = miniexp_car(r);
     }
   if (r) 
     return false;
-  if (comma.isEmpty() && !nospace.contains(type))
-    comma = " ";
+  state = qMax(state, typenum);
   return true;
 }
 
@@ -1151,10 +1170,8 @@ miniexp_search_text(miniexp_t exp, QRegExp regex)
   QString text;
   QMap<int, miniexp_t> positions;
   // build string
-  QList<miniexp_t> nospace;
-  QString comma;
-  nospace << miniexp_symbol("char");
-  if (! miniexp_get_text(exp, text, positions, nospace, comma))
+  int state = -1;
+  if (! miniexp_get_text(exp, text, positions, state))
     return hits;
   // search hits
   int offset = 0;
@@ -1163,10 +1180,13 @@ miniexp_search_text(miniexp_t exp, QRegExp regex)
     {
       QList<miniexp_t> hit;
       int endmatch = match + regex.matchedLength();
+      offset += 1;
+      if (endmatch <= match)
+        continue;
       QMap<int,miniexp_t>::const_iterator pos = positions.lowerBound(match);
       while (pos != positions.begin() && pos.key() > match)
         --pos;
-      for (; pos!=positions.end() && pos.key() < endmatch; ++pos)
+      for (; pos != positions.end() && pos.key() < endmatch; ++pos)
         hit += pos.value();
       hits += hit;
       if (pos != positions.end())
@@ -1354,6 +1374,10 @@ QDjViewFind::Model::workTimeout()
                            "No page contains information "
                            "about its textual content.</html>");
                 }
+              else if (! find.isValid())
+                {
+                  msg = tr("<html>Invalid regular expression.</html>");
+                }
               widget->stack->setCurrentWidget(widget->label);
               widget->label->setText(msg);
             }
@@ -1405,7 +1429,7 @@ QDjViewFind::Model::startFind(bool backwards, int delay)
   if (! find.isEmpty() && djview->pageNum() > 0)
     {
       widget->label->setText(QString());
-      widget->stack->setCurrentIndex(0);
+      widget->stack->setCurrentWidget(widget->view);
       animButton = (backwards) ? widget->upButton : widget->downButton;
       animIcon = animButton->icon();
       animTimer->start(250);
@@ -1467,15 +1491,18 @@ QDjViewFind::Model::textChanged()
   clear();
   QString s = widget->text();
   widget->label->setText(QString());
-  widget->stack->setCurrentIndex(0);
+  widget->stack->setCurrentWidget(widget->label);
   if (s.isEmpty())
     {
       find = QRegExp();
     }
   else
     {
-      s = QRegExp::escape(widget->text());
-      s.replace(QRegExp("\\s+"), " ");
+      if (!regExpMode)
+        {
+          s = QRegExp::escape(widget->text());
+          s.replace(QRegExp("\\s+"), " ");
+        }
       if (wordOnly)
         s = "\\b" + s;
       find = QRegExp(s);
@@ -1525,43 +1552,51 @@ QDjViewFind::QDjViewFind(QDjView *djview)
   view->setViewMode(QListView::ListMode);
   view->setWrapping(false);
   view->setResizeMode(QListView::Adjust);
-  
-  QAction *eraseAction = new QAction(tr("Erase text"), this);
-  eraseAction->setIcon(QIcon(":/images/icon_erase.png"));
   caseSensitiveAction = new QAction(tr("Case sensitive"), this);
   caseSensitiveAction->setCheckable(true);
   caseSensitiveAction->setChecked(model->caseSensitive);
   wordOnlyAction = new QAction(tr("Words only"), this);
   wordOnlyAction->setCheckable(true);
   wordOnlyAction->setChecked(model->wordOnly);
+  regExpModeAction = new QAction(tr("Regular expression"), this);
+  regExpModeAction->setCheckable(true);
+  regExpModeAction->setChecked(model->regExpMode);
   menu = new QMenu(this);
   menu->addAction(caseSensitiveAction);
   menu->addAction(wordOnlyAction);
+  menu->addAction(regExpModeAction);
   QBoxLayout *vlayout = new QVBoxLayout(this);
-  QToolBar *tools = new QToolBar(this);
-  tools->addAction(eraseAction);
-  combo = new QComboBox(tools);
+  combo = new QComboBox(this);
   combo->setEditable(true);
   combo->setMaxCount(8);
   combo->setInsertPolicy(QComboBox::InsertAtTop);
   combo->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
-  tools->addWidget(combo);
-  tools->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Minimum);
-  vlayout->addWidget(tools);
+  vlayout->addWidget(combo);
   QBoxLayout *hlayout = new QHBoxLayout;
+  hlayout->setSpacing(0);
   vlayout->addLayout(hlayout);
-  upButton = new QPushButton(this);
+  upButton = new QToolButton(this);
   upButton->setIcon(QIcon(":/images/icon_up.png"));
   upButton->setToolTip(tr("Find Previous (Shift+F3) "));
+  upButton->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
   hlayout->addWidget(upButton);
-  downButton = new QPushButton(this);
+  downButton = new QToolButton(this);
   downButton->setIcon(QIcon(":/images/icon_down.png"));
   downButton->setToolTip(tr("Find Next (F3) "));
+  downButton->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
   hlayout->addWidget(downButton);
   hlayout->addStretch(2);
-  QPushButton *optionButton = new QPushButton(tr("Options"), this);
+  resetButton = new QToolButton(this);
+  resetButton->setIcon(QIcon(":/images/icon_erase.png"));
+  resetButton->setToolTip(tr("Reset search options to default values."));
+  resetButton->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
+  hlayout->addWidget(resetButton);
+  QToolButton *optionButton = new QToolButton(this);
+  optionButton->setText(tr("Options"));
+  optionButton->setPopupMode(QToolButton::InstantPopup);
+  optionButton->setToolButtonStyle(Qt::ToolButtonTextOnly);
   optionButton->setMenu(menu);
-  optionButton->setFlat(true);
+  optionButton->setAttribute(Qt::WA_CustomWhatsThis);
   optionButton->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
   hlayout->addWidget(optionButton);
   stack = new QStackedLayout();
@@ -1590,27 +1625,66 @@ QDjViewFind::QDjViewFind(QDjView *djview)
           model, SLOT(textChanged()));
   connect(combo->lineEdit(), SIGNAL(returnPressed()),
           this, SLOT(findAgain()));
-  connect(eraseAction, SIGNAL(triggered()), 
-          this, SLOT(eraseText()));
   connect(caseSensitiveAction, SIGNAL(triggered(bool)),
           this, SLOT(setCaseSensitive(bool)));
   connect(wordOnlyAction, SIGNAL(triggered(bool)),
           this, SLOT(setWordOnly(bool)));
+  connect(regExpModeAction, SIGNAL(triggered(bool)),
+          this, SLOT(setRegExpMode(bool)));
   connect(upButton, SIGNAL(clicked()), 
           this, SLOT(findPrev()));
   connect(downButton, SIGNAL(clicked()), 
           this, SLOT(findNext()));
+  connect(resetButton, SIGNAL(clicked()), 
+          this, SLOT(eraseText()));
   
-  setWhatsThis(tr("<html><b>Finding text.</b><br/> "
-                  "Search hits appear progressively as soon as you type "
-                  "a search string. Typing enter jumps to the next hit. "
-                  "To move to the previous or next hit, you can also use "
-                  "the arrow buttons or the shortcuts <tt>F3</tt> or "
-                  "<tt>Shift-F3</tt>. You can also double click a page name. "
-                  "Use the <tt>Options</tt> menu to search words only or "
-                  "to specify the case sensitivity."
-                  "</html>"));
+  setWhatsThis
+    (tr("<html><b>Finding text.</b><br/> "
+        "Search hits appear progressively as soon as you type "
+        "a search string. Typing enter jumps to the next hit. "
+        "To move to the previous or next hit, you can also use "
+        "the arrow buttons or the shortcuts <tt>F3</tt> or "
+        "<tt>Shift-F3</tt>. You can also double click a page name. "
+        "Use the <tt>Options</tt> menu to search words only or "
+        "to specify the case sensitivity."
+        "</html>"));
+  wordOnlyAction->setStatusTip
+    (tr("Specify whether search hits must begin on a word boundary."));
+  caseSensitiveAction->setStatusTip
+    (tr("Specify whether searches are case sensitive."));
+  regExpModeAction->setStatusTip
+    (tr("Regular expressions describe complex string matching patterns."));
+  regExpModeAction->setWhatsThis
+    (tr("<html><b>Regular Expression Quick Guide</b><ul>"
+        "<li>The dot <tt>.</tt> matches any character.</li>"
+        "<li>Most characters match themselves.</li>"
+        "<li>Prepend a backslash <tt>\\</tt> to match special"
+        "    characters <tt>()[]{}|*+.?!^$\\</tt>.</li>"
+        "<li><tt>\\b</tt> matches a word boundary.</li>"
+        "<li><tt>\\w</tt> matches a word character.</li>"
+        "<li><tt>\\d</tt> matches a digit character.</li>"
+        "<li><tt>\\s</tt> matches a blank character.</li>"
+        "<li><tt>\\n</tt> matches a newline character.</li>"
+        "<li><tt>[<i>a</i>-<i>b</i>]</tt> matches characters in range"
+        "    <tt><i>a</i></tt>-<tt><i>b</i></tt>.</li>"
+        "<li><tt>[^<i>a</i>-<i>b</i>]</tt> matches characters outside range"
+        "    <tt><i>a</i></tt>-<tt><i>b</i></tt>.</li>"
+        "<li><tt><i>a</i>|<i>b</i></tt> matches either regular expression"
+        "    <tt><i>a</i></tt> or regular expression <tt><i>b</i></tt>.</li>"
+        "<li><tt><i>a</i>{<i>n</i>,<i>m</i>}</tt> matches regular expression"
+        "    <tt><i>a</i></tt> repeated <tt><i>n</i></tt> to <tt><i>m</i></tt>"
+        "    times.</li>"
+        "<li><tt><i>a</i>?</tt>, <tt><i>a</i>*</tt>, and <tt><i>a</i>+</tt>"
+        "    are shorthands for <tt><i>a</i>{0,1}</tt>, <tt><i>a</i>{0,}</tt>, "
+        "    and <tt><i>a</i>{1,}</tt>.</li>"
+        "<li>Use parentheses <tt>()</tt> to group regular expressions "
+        "    before <tt>?+*{</tt>.</li>"
+        "</ul></html>"));
 
+  eraseText();
+  combo->setEnabled(false);
+  label->setText(QString());
+  stack->setCurrentWidget(label);
   if (djview->getDocument())
     model->documentReady(djview->getDocument());
 }
@@ -1653,6 +1727,13 @@ QDjViewFind::wordOnly()
 }
 
 
+bool 
+QDjViewFind::regExpMode()
+{
+  return model->regExpMode;
+}
+
+
 void 
 QDjViewFind::setText(QString s)
 {
@@ -1672,7 +1753,10 @@ QDjViewFind::selectAll()
 void 
 QDjViewFind::eraseText()
 {
-  setText(QString());
+  setText(QString::null);
+  setRegExpMode(false);
+  setWordOnly(true);
+  setCaseSensitive(false);
 }
 
 
@@ -1700,6 +1784,18 @@ QDjViewFind::setWordOnly(bool b)
 }
 
 
+void 
+QDjViewFind::setRegExpMode(bool b)
+{
+  if (b != model->regExpMode)
+    {
+      regExpModeAction->setChecked(b);
+      model->regExpMode = b;
+      model->textChanged();
+    }
+}
+
+
 void 
 QDjViewFind::findNext()
 {
