@@ -1,7 +1,7 @@
 //C-  -*- C++ -*-
 //C- -------------------------------------------------------------------
 //C- DjView4
-//C- Copyright (c) 2006  Leon Bottou
+//C- Copyright (c) 2006-  Leon Bottou
 //C-
 //C- This software is subject to, and may be distributed under, the
 //C- GNU General Public License, either version 2 of the license,
@@ -15,7 +15,7 @@
 //C- GNU General Public License for more details.
 //C-  ------------------------------------------------------------------
 
-// $Id: qdjvuwidget.h,v 1.34 2008/08/08 11:37:46 leonb Exp $
+// $Id: qdjvuwidget.h,v 1.40 2009/02/22 16:50:54 leonb Exp $
 
 #ifndef QDJVUWIDGET_H
 #define QDJVUWIDGET_H
@@ -47,7 +47,7 @@ class QDjVuWidget : public QAbstractScrollArea
              READ zoom WRITE setZoom)
   Q_PROPERTY(double gamma 
              READ gamma WRITE setGamma)
-  Q_PROPERTY(double screenDpi 
+  Q_PROPERTY(int screenDpi 
              READ screenDpi WRITE setScreenDpi)
   Q_PROPERTY(DisplayMode displayMode 
              READ displayMode WRITE setDisplayMode)
@@ -113,8 +113,9 @@ public:
   enum DisplayMode {
     DISPLAY_COLOR,              //!< Default dislplay mode
     DISPLAY_STENCIL,            //!< Only display the b&w mask layer
-    DISPLAY_BG,                 //!< Onlu display the background layer
-    DISPLAY_FG,                 //!< Onlu display the foregroud layer
+    DISPLAY_BG,                 //!< Only display the background layer
+    DISPLAY_FG,                 //!< Only display the foregroud layer
+    DISPLAY_TEXT,               //!< Overprint the text layer
   };
 
   enum Align {
@@ -129,7 +130,7 @@ public:
     PRIORITY_DEFAULT,           //!< Priority for default option values.
     PRIORITY_ANNO,              //!< Priority for annotation defined options.
     PRIORITY_CGI,               //!< Priority for cgi defined options.
-    PRIORITY_USER               //!< Priority for gui defined options.
+    PRIORITY_USER,              //!< Priority for gui defined options.
   };  
 
   struct Position {
@@ -162,6 +163,7 @@ public:
   QPoint hotSpot(void) const;
   Position position(void) const;
   Position position(const QPoint &point) const;
+  Position positionWithClosestAnchor(const QPoint &point) const;
   int rotation(void) const;
   int zoom(void) const;
   int zoomFactor(void) const;
@@ -278,6 +280,7 @@ public slots:
   void displayModeStencil(void);
   void displayModeBackground(void);
   void displayModeForeground(void);
+  void displayModeText(void);
   void rotateRight(void);
   void rotateLeft(void);
 
